package org.hashtree.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import org.hashtree.jbrainhoney.dlap.element.Enrollment;

/**
 * A command which enrolls a user with the specified rights in a course or section. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateEnrollments">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class CreateEnrollmentsCommand implements Command {
	private final List<Enrollment> enrollments;

	@SuppressWarnings("synthetic-access")
	private CreateEnrollmentsCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.enrollments == null || builder.enrollments.size() == 0)
			throw new IllegalArgumentException("Expected enrollments to not be null.");

		for(Enrollment enrollment : builder.enrollments) {
			if(enrollment == null) throw new IllegalArgumentException("Expected enrollment to not be null.");
			else if(enrollment.getFlags() == null || enrollment.getEndDate() == null || enrollment.getEntityId() == null
				|| enrollment.getStartDate() == null || enrollment.getStatus() == null || enrollment.getUserId() == null) {

				throw new IllegalArgumentException("Expected flags, endDate, entityId, startDate, status, and userId to all not be null.");
			}
		}

		enrollments = Collections.unmodifiableList(new ArrayList<Enrollment>(builder.enrollments));
	}

	public List<Enrollment> getEnrollments() {
		return enrollments;
	}

	/** A builder of immutable CreateEnrollmentsCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Enrollment> enrollments;

		public Builder(final List<Enrollment> enrollments) {
			super();

			if(enrollments == null) this.enrollments = Collections.synchronizedList(new ArrayList<Enrollment>());
			else this.enrollments = Collections.synchronizedList(new ArrayList<Enrollment>(enrollments));
		}

		public Builder(final Enrollment enrollment) {
			super();

			if(enrollment == null) enrollments = Collections.synchronizedList(new ArrayList<Enrollment>());
			else enrollments = Collections.synchronizedList(new ArrayList<Enrollment>(Arrays.asList(enrollment)));
		}

		public Builder addEnrollment(final Enrollment enrollment) {
			enrollments.add(enrollment);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public CreateEnrollmentsCommand build() {
			return new CreateEnrollmentsCommand(this);
		}
	}
}