package org.hashtree.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import org.hashtree.jbrainhoney.dlap.element.Group;

/**
 * A command which creates one or more groups in the specified owner group. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/Creategroups">the BrainHoney DLAP API guide</a>} for this command.
 */
public final class CreateGroupsCommand implements Command {
	private final List<Group> groups;

	@SuppressWarnings("synthetic-access")
	private CreateGroupsCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.groups == null || builder.groups.size() == 0)
			throw new IllegalArgumentException("Expected groups to not be null.");

		for(Group group : builder.groups) {
			if(group == null) throw new IllegalArgumentException("Expected group to not be null.");
			else if(group.getDomainId() == null || group.getOwnerId() == null || group.getSetId() == null
				|| group.getTitle() == null) {

				throw new IllegalArgumentException("Expected domainId, ownerId, setId, and title to all not be null.");
			}
		}

		groups = Collections.unmodifiableList(new ArrayList<Group>(builder.groups));
	}

	public List<Group> getGroups() {
		return groups;
	}

	/** A builder of immutable CreateGroupsCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Group> groups;

		public Builder(final List<Group> groups) {
			super();

			if(groups == null) this.groups = Collections.synchronizedList(new ArrayList<Group>());
			else this.groups = Collections.synchronizedList(new ArrayList<Group>(groups));
		}

		public Builder(final Group group) {
			super();

			if(group == null) groups = Collections.synchronizedList(new ArrayList<Group>());
			else groups = Collections.synchronizedList(new ArrayList<Group>(Arrays.asList(group)));
		}

		public Builder addGroup(final Group group) {
			groups.add(group);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public CreateGroupsCommand build() {
			return new CreateGroupsCommand(this);
		}
	}
}