package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.UserspaceAttributeValidator;

/**
 * A command which authenticates a user on the server and initiates a session state using a SAML assertion for single
 * sign-on. See the {@link <a href="http://gls.agilix.com/Docs/Command/SamlLogin">the BrainHoney DLAP API guide</a>}
 * for this command.
 */
public final class SamlLoginCommand implements Command, ObservableSessionCommand {
	private final String response;
	private final SessionObserverManager sessionObserverManager;
	private final String userspace;

	@SuppressWarnings("synthetic-access")
	private SamlLoginCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.response == null) throw new IllegalArgumentException("Expected response to not be null.");
		else if(builder.sessionObserverManager == null)
			throw new IllegalArgumentException("Expected sessionObserverManager to not be null.");
		else if(builder.userspace == null) throw new IllegalArgumentException("Expected userspace to not be null.");
		else if(!UserspaceAttributeValidator.getInstance().isValid(builder.userspace))
			throw new IllegalArgumentException("Expected userspace to be valid.");

		response = builder.response;
		sessionObserverManager = builder.sessionObserverManager;
		userspace = builder.userspace;
	}

	public String getResponse() {
		return response;
	}

	public SessionObserverManager getSessionObserverManager() {
		return sessionObserverManager;
	}

	public String getUserspace() {
		return userspace;
	}

	@Override
	public SamlLoginCommand addObserver(final SessionObserver observer) {
		sessionObserverManager.addObserver(observer);

		return this;
	}

	@Override
	public SamlLoginCommand removeObserver(final SessionObserver observer) {
		sessionObserverManager.removeObserver(observer);

		return this;
	}

	/** A builder of immutable SamlLoginCommand instances. */
	public final static class Builder implements Command.Builder, ObservableSessionCommand.Builder {
		private final String response;
		private final SessionObserverManager sessionObserverManager;
		private final String userspace;

		public Builder(final String userspace, final String response) {
			super();

			this.response = response;
			this.sessionObserverManager = new SessionObserverManager();
			this.userspace = userspace;
		}

		@Override
		public Builder addObserver(final SessionObserver observer) {
			sessionObserverManager.addObserver(observer);

			return this;
		}

		@Override
		public Builder removeObserver(final SessionObserver observer) {
			sessionObserverManager.removeObserver(observer);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public SamlLoginCommand build() {
			return new SamlLoginCommand(this);
		}
	}
}