package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which authenticates a user on the server and initiates a session state using a SSO provider. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/SSOLogin">the BrainHoney DLAP API guide</a>} for this command.
 */
public final class SsoLoginCommand implements Command, ObservableSessionCommand {
	private final String domainId;
	private final SessionObserverManager sessionObserverManager;
	private final String service;
	private final String system;
	private final String ticket;

	@SuppressWarnings("synthetic-access")
	private SsoLoginCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.domainId == null) throw new IllegalArgumentException("Expected domainId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.sessionObserverManager == null)
			throw new IllegalArgumentException("Expected sessionObserverManager to not be null.");
		else if(builder.service == null) throw new IllegalArgumentException("Expected service to not be null.");
		else if(builder.system == null) throw new IllegalArgumentException("Expected system to not be null.");
		else if(builder.ticket == null) throw new IllegalArgumentException("Expected ticket to not be null.");

		domainId = builder.domainId;
		sessionObserverManager = builder.sessionObserverManager;
		service = builder.service;
		system = builder.system;
		ticket = builder.ticket;
	}

	public String getDomainId() {
		return domainId;
	}

	public SessionObserverManager getSessionObserverManager() {
		return sessionObserverManager;
	}

	public String getService() {
		return service;
	}

	public String getSystem() {
		return system;
	}

	public String getTicket() {
		return ticket;
	}

	@Override
	public SsoLoginCommand addObserver(final SessionObserver observer) {
		sessionObserverManager.addObserver(observer);

		return this;
	}

	@Override
	public SsoLoginCommand removeObserver(final SessionObserver observer) {
		sessionObserverManager.removeObserver(observer);

		return this;
	}

	/** A builder of immutable SsoLoginCommand instances. */
	public final static class Builder implements Command.Builder, ObservableSessionCommand.Builder {
		private final String domainId;
		private final SessionObserverManager sessionObserverManager;
		private final String service;
		private final String system;
		private final String ticket;

		public Builder(final String system, final String domainId, final String ticket, final String service) {
			super();

			this.domainId = domainId;
			this.sessionObserverManager = new SessionObserverManager();
			this.service = service;
			this.system = system;
			this.ticket = ticket;
		}

		public Builder(final String system, final Integer domainId, final String ticket, final String service) {
			super();

			this.domainId = (domainId == null ? null : domainId.toString());
			this.sessionObserverManager = new SessionObserverManager();
			this.service = service;
			this.system = system;
			this.ticket = ticket;
		}

		@Override
		public Builder addObserver(final SessionObserver observer) {
			sessionObserverManager.addObserver(observer);

			return this;
		}

		@Override
		public Builder removeObserver(final SessionObserver observer) {
			sessionObserverManager.removeObserver(observer);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public SsoLoginCommand build() {
			return new SsoLoginCommand(this);
		}
	}
}