package org.hashtree.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import org.hashtree.jbrainhoney.dlap.element.Section;

/**
 * A command which updates the title, reference, type, startDate, endDate, and days attributes of one or more sections.
 * It updates only the specified attributes. Any omitted attributes retain their existing values. You can also change
 * a section from one course to another. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateSections">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class UpdateSectionsCommand implements Command {
	private final List<Section> sections;

	@SuppressWarnings("synthetic-access")
	private UpdateSectionsCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.sections == null || builder.sections.size() == 0)
			throw new IllegalArgumentException("Expected sections to not be null.");

		for(Section section : builder.sections) {
			if(section == null) throw new IllegalArgumentException("Expected section to not be null.");
			else if(section.getSectionId() == null || (section.getCourseId() == null && section.getData() == null
				&& section.getDays() == null && section.getEndDate() == null && section.getStartDate() == null
				&& section.getReference() == null && section.getTerm() == null && section.getTitle() == null
				&& section.getType() == null))

				throw new IllegalArgumentException("Expected sectionId and courseId, data, days, endDate, startDate, reference, term, title, or type to not be null.");
		}

		sections = Collections.unmodifiableList(new ArrayList<Section>(builder.sections));
	}

	public List<Section> getSections() {
		return sections;
	}

	/** A builder of immutable UpdateSectionsCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Section> sections;

		public Builder(final List<Section> sections) {
			super();

			if(sections == null) this.sections = Collections.synchronizedList(new ArrayList<Section>());
			else this.sections = Collections.synchronizedList(new ArrayList<Section>(sections));
		}

		public Builder(final Section section) {
			super();

			if(section == null) sections = Collections.synchronizedList(new ArrayList<Section>());
			else sections = Collections.synchronizedList(new ArrayList<Section>(Arrays.asList(section)));
		}

		public Builder addSection(final Section section) {
			sections.add(section);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public UpdateSectionsCommand build() {
			return new UpdateSectionsCommand(this);
		}
	}
}