package org.hashtree.jbrainhoney.dlap.element;

import java.util.*;
import nu.xom.Attribute;
import nu.xom.Element;
import org.hashtree.jbrainhoney.compare.NullComparatorUtility;
import org.hashtree.jbrainhoney.dlap.enumeration.EmailRecipient;
import org.hashtree.jbrainhoney.dlap.enumeration.RightFlag;

/** An email. */
public final class Email implements XmlElement, Comparable<Email> {
	private static final long serialVersionUID = 536471820565829221L;
	private final String subject;
	private final String body;
	private final List<Enrollment> enrollments;
	private final List<Group> groups;
	private final List<RightFlag> roles;
	private final EmailRecipient recipients;
	private final Map<String, String> strings;

	@SuppressWarnings("synthetic-access")
	private Email(final Builder builder) throws IllegalArgumentException {
		if(builder.subject == null) throw new IllegalArgumentException("Expected subject to not be null.");

		if(builder.enrollments != null) {
			for(Enrollment enrollment : builder.enrollments) {
				if(enrollment == null) throw new IllegalArgumentException("Expected enrollment to not be null.");
				else if(enrollment.getEnrollmentId() == null)
					throw new IllegalArgumentException("Expected enrollmentId to not be null.");
			}
		}

		if(builder.groups != null)
			for(Group group : builder.groups) {
				if(group == null) throw new IllegalArgumentException("Expected group to not be null.");
				else if(group.getGroupId() == null)
					throw new IllegalArgumentException("Expected groupId to not be null.");
			}

		if(builder.roles != null)
			for(RightFlag role : builder.roles) {
				if(role == null) throw new IllegalArgumentException("Expected group to not be null.");
			}

		if(builder.strings != null) {
			List<String> validStrings = new ArrayList<String>();
			validStrings.add("invalidAddress");
			validStrings.add("noRecipients");
			validStrings.add("failedRecipients");
			validStrings.add("deliveryFailure");
			validStrings.add("failureSubject");
			validStrings.add("failureBody");
			validStrings.add("fullNameFormat");

			for(String stringKey : builder.strings.keySet()) {
				if(!validStrings.contains(stringKey))
					throw new IllegalArgumentException("Expected valid string element.");
			}
		}

		subject = builder.subject;
		body = builder.body;
		enrollments = (builder.enrollments != null ? Collections.unmodifiableList(new ArrayList<Enrollment>(builder.enrollments)) : null);
		groups = (builder.groups != null ? Collections.unmodifiableList(new ArrayList<Group>(builder.groups)) : null);
		recipients = (builder.recipients != null ? builder.recipients : null);
		roles = (builder.roles != null ? Collections.unmodifiableList(new ArrayList<RightFlag>(builder.roles)) : null);
		strings = (builder.strings != null ? Collections.unmodifiableMap(new HashMap<String, String>(builder.strings)) : null);
	}

	@Override
	public int compareTo(final Email email) throws NullPointerException {
		if(email == null) throw new NullPointerException("Expected email to not be null.");

		return NullComparatorUtility.compareIgnoreNullAndCase(this.subject, email.getSubject());
	}

	public String getSubject() {
		return subject;
	}

	public String getBody() {
		return body;
	}

	public List<Enrollment> getEnrollments() {
		return enrollments;
	}

	public List<Group> getGroups() {
		return groups;
	}

	public EmailRecipient getRecipients() {
		return recipients;
	}

	public List<RightFlag> getRoles() {
		return roles;
	}

	public Map<String, String> getStrings() {
		return strings;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Email)) return false;

		return toString().equals(((Email)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(Email.class.getSimpleName() + "(");
		if(body != null) stringBuilder.append("body=" + body + ", ");
		if(enrollments != null) stringBuilder.append("enrollments=" + enrollments.toString() + ", ");
		if(groups != null) stringBuilder.append("groups=" + groups.toString() + ", ");
		if(recipients != null) stringBuilder.append("recipients=" + recipients.getValue() + ", ");
		if(roles != null) stringBuilder.append("roles=" + roles.toString() + ", ");
		if(strings != null) stringBuilder.append("strings=" + strings.toString() + ", ");
		if(subject != null) stringBuilder.append("subject=" + subject + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("email");

		if(body != null) {
			Element bodyElement = new Element("body");
			bodyElement.appendChild(body);
			element.appendChild(bodyElement);
		}

		if(recipients != null || (enrollments != null && enrollments.size() > 0)) {
			Element enrollmentsElement = new Element("enrollments");

			if(recipients != null) {
				Element enrollmentElement = new Element("enrollment");
				enrollmentElement.addAttribute(new Attribute("id", recipients.getValue().toLowerCase()));
				enrollmentsElement.appendChild(enrollmentElement);
			}

			if(enrollments != null && enrollments.size() > 0) {
				for(Enrollment enrollment : enrollments) {
					Element enrollmentElement = new Element("enrollment");
					enrollmentElement.addAttribute(new Attribute("id", enrollment.getEnrollmentId().toString()));
					enrollmentsElement.appendChild(enrollmentElement);
				}
			}

			element.appendChild(enrollmentsElement);
		}

		if(groups != null && groups.size() > 0) {
			Element groupsElement = new Element("groups");

			for(Group group : groups) {
				Element groupElement = new Element("group");
				groupElement.addAttribute(new Attribute("id", group.getGroupId().toString()));
				groupsElement.appendChild(groupElement);
			}

			element.appendChild(groupsElement);
		}

		if(roles != null && roles.size() > 0) {
			Element rolesElement = new Element("roles");

			for(RightFlag role : roles) {
				Element roleElement = new Element("role");
				roleElement.addAttribute(new Attribute("flags", role.getValue().toString()));
				rolesElement.appendChild(roleElement);
			}

			element.appendChild(rolesElement);
		}

		if(strings != null && strings.size() > 0) {
			Element stringsElement = new Element("strings");

			for(String stringKey : strings.keySet()) {
				Element stringElement = new Element(stringKey.toLowerCase());
				stringElement.appendChild(strings.get(stringKey));
				stringsElement.appendChild(stringElement.copy());
			}

			element.appendChild(stringsElement);
		}

		Element subjectElement = new Element("subject");
		subjectElement.appendChild(subject);
		element.appendChild(subjectElement);

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable Email elements. */
	public final static class Builder implements XmlElement.Builder {
		private final String subject;
		private String body;
		private List<Enrollment> enrollments;
		private List<Group> groups;
		private EmailRecipient recipients;
		private List<RightFlag> roles;
		private Map<String, String> strings;

		public Builder(final String subject) {
			this.subject = subject;
		}

		public Builder setBody(final String body) {
			this.body = body;

			return this;
		}

		public Builder setEnrollments(final List<Enrollment> enrollments) {
			this.enrollments = (enrollments == null ? null : Collections.synchronizedList(new ArrayList<Enrollment>(enrollments)));

			return this;
		}

		public Builder setEnrollments(final Enrollment enrollment) {
			enrollments = (enrollment == null ? null : Collections.synchronizedList(new ArrayList<Enrollment>(Arrays.asList(enrollment))));

			return this;
		}

		public Builder addEnrollment(final Enrollment enrollment) {
			if(enrollments == null)
				enrollments = Collections.synchronizedList(new ArrayList<Enrollment>());

			enrollments.add(enrollment);

			return this;
		}

		public Builder setGroups(final List<Group> groups) {
			this.groups = (groups == null ? null : Collections.synchronizedList(new ArrayList<Group>(groups)));

			return this;
		}

		public Builder setGroups(final Group group) {
			groups = (group == null ? null : Collections.synchronizedList(new ArrayList<Group>(Arrays.asList(group))));

			return this;
		}

		public Builder addGroup(final Group group) {
			if(groups == null) groups = Collections.synchronizedList(new ArrayList<Group>());

			groups.add(group);

			return this;
		}

		public Builder setRecipients(final EmailRecipient recipients) {
			this.recipients = recipients;

			return this;
		}

		public Builder setRoles(final List<RightFlag> roles) {
			this.roles = (roles == null ? null : Collections.synchronizedList(new ArrayList<RightFlag>(roles)));

			return this;
		}

		public Builder setRoles(final RightFlag role) {
			roles = (role == null ? null : Collections.synchronizedList(new ArrayList<RightFlag>(Arrays.asList(role))));

			return this;
		}

		public Builder addRole(final RightFlag role) {
			if(roles == null) roles = Collections.synchronizedList(new ArrayList<RightFlag>());

			roles.add(role);

			return this;
		}

		public Builder setStrings(final Map<String, String> strings) {
			this.strings = (strings == null ? null : Collections.synchronizedMap(new HashMap<String, String>(strings)));

			return this;
		}

		public Builder putString(final String key, final String value) {
			if(strings == null) strings = Collections.synchronizedMap(new HashMap<String, String>());

			strings.put(key, value);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public Email build() {
			return new Email(this);
		}
	}
}