package org.hashtree.jbrainhoney.dlap.parse;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import nu.xom.ValidityException;
import org.hashtree.jbrainhoney.dlap.DlapException;
import org.hashtree.jbrainhoney.dlap.ResponseCode;
import org.hashtree.jbrainhoney.dlap.element.*;
import org.hashtree.jbrainhoney.dlap.enumeration.EntityType;

/** A XML based parser. */
public final class XmlParser implements Parser
{
	private XmlParser() {

	}

	public static Element parseRootElement(final String xml) throws IllegalArgumentException, DlapException {
		nu.xom.Builder builder = new nu.xom.Builder();

		if(xml == null || xml.trim().length() == 0) throw new IllegalArgumentException("Expected XML.");

		try {
			Document parsedDocument = builder.build(xml, null);

			if(parsedDocument.getRootElement() == null) throw new IllegalArgumentException("Expected XML root element.");

			return parsedDocument.getRootElement();
		} catch(ValidityException exception) {
			throw new DlapException(exception);
		} catch(ParsingException exception) {
			throw new DlapException(exception);
		} catch(IOException exception) {
			throw new DlapException(exception);
		}
	}

	private static Element parseXmlElement(final Elements elements, final String singularQualifiedName)
		throws IllegalArgumentException {

		if(elements != null && elements.size() > 0) {
			Element parsedElement = elements.get(0);

			if(!parsedElement.getQualifiedName().equals(singularQualifiedName))
				throw new IllegalArgumentException("Expected well formed XML " + singularQualifiedName + " element.");

			return parsedElement;
		}

		return null;
	}

	private static List<Element> parseXmlElements(final Elements elements, final String pluralQualifiedName, final String singularQualifiedName)
		throws IllegalArgumentException {

		if(elements != null && elements.size() > 0) {
			if(!elements.get(0).getQualifiedName().equals(pluralQualifiedName))
				throw new IllegalArgumentException("Expected XML element.");

			Elements parsedElements = elements.get(0).getChildElements();
			Element parsedElement = null;
			List<Element> parsedXmlElements = new ArrayList<Element>(parsedElements.size());

			for(int i = 0; i < parsedElements.size(); i++) {
				parsedElement = parsedElements.get(i);

				if(parsedElement.getQualifiedName().equals(singularQualifiedName)) parsedXmlElements.add(parsedElement);
				else throw new IllegalArgumentException("Expected well formed XML " + singularQualifiedName + " element.");
			}

			return Collections.synchronizedList(parsedXmlElements);
		}

		return Collections.synchronizedList(new ArrayList<Element>(0));
	}

	private static List<Element> parseNestedXmlElements(final Elements elements, final String singularQualifiedName)
		throws IllegalArgumentException {

		if(elements != null && elements.size() > 0) {
			if(!elements.get(0).getQualifiedName().equals("responses"))
				throw new IllegalArgumentException("Expected XML element.");

			Elements parsedElements = elements.get(0).getChildElements();
			Element parsedElement = null;
			List<Element> parsedXmlElements = new ArrayList<Element>();
			ResponseCode responseCode = null;

			for(int i = 0; i < parsedElements.size(); i++) {
				parsedElement = parsedElements.get(i);

				try {
					responseCode = ResponseCode.valueOf(parsedElement.getAttribute("code").getValue().toUpperCase());
				} catch(IllegalArgumentException exception) {
					responseCode = ResponseCode.EXCEPTION;
				}

				// Only add when there is a valid response code.
				if(parsedElement.getAttribute("code") != null && responseCode.getExceptionClass() == null
					&& parsedElement.getFirstChildElement(singularQualifiedName) != null) {

					parsedXmlElements.add(parsedElement.getFirstChildElement(singularQualifiedName));
				}
			}

			return Collections.synchronizedList(parsedXmlElements);
		}

		return Collections.synchronizedList(new ArrayList<Element>(0));
	}

	public static List<ResponseCode> parseResponseCodes(final Elements elements) throws IllegalArgumentException {
		if(elements != null && elements.size() > 0) {
			if(!elements.get(0).getQualifiedName().equals("responses"))
				throw new IllegalArgumentException("Expected XML element");

			Elements responseElements = elements.get(0).getChildElements();
			Element responseElement = null;
			List<ResponseCode> responseCodes = new ArrayList<ResponseCode>(responseElements.size());

			for(int i = 0; i < responseElements.size(); i++) {
				responseElement = responseElements.get(i);

				if(responseElement.getQualifiedName().equals("response") && responseElement.getAttribute("code") != null) {
					try {
						responseCodes.add(ResponseCode.valueOf(responseElement.getAttribute("code").getValue().toUpperCase()));
					} catch(IllegalArgumentException exception) {
						responseCodes.add(ResponseCode.EXCEPTION);
					}
				} else throw new IllegalArgumentException("Expected well formed XML reponse element.");
			}

			return Collections.synchronizedList(responseCodes);
		}

		return Collections.synchronizedList(new ArrayList<ResponseCode>(0));
	}

	public static EntityType parseEntityType(final Elements elements) throws IllegalArgumentException {
		if(elements != null && elements.size() > 0) {
			Element entityElement = elements.get(0);

			if(!entityElement.getQualifiedName().equals("entity") || entityElement.getAttribute("entitytype") == null)
				throw new IllegalArgumentException("Expected well formed XML entity element.");

			try {
				return EntityType.valueOf(entityElement.getAttribute("entitytype").getValue().toUpperCase());
			} catch(IllegalArgumentException exception) {
				throw exception;
			}
		}

		return null;
	}

	public static List<Domain> parseNestedDomains(final Elements elements) {
		List<Element> parsedElements = parseNestedXmlElements(elements, "domain");
		List<Domain> domains = new ArrayList<Domain>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) domains.add(new Domain(parsedElement));
		}

		return Collections.synchronizedList(domains);
	}

	public static List<Domain> parseDomains(final Elements elements) {
		List<Element> parsedElements = parseXmlElements(elements, "domains", "domain");
		List<Domain> domains = new ArrayList<Domain>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) domains.add(new Domain(parsedElement));
		}

		return Collections.synchronizedList(domains);
	}

	public static Domain parseDomain(final Elements elements) {
		Element parsedElement = parseXmlElement(elements, "domain");

		if(parsedElement == null) return null;

		return new Domain(parsedElement);
	}

	public static List<User> parseNestedUsers(final Elements elements) {
		List<Element> parsedElements = parseNestedXmlElements(elements, "user");
		List<User> users = new ArrayList<User>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) users.add(new User(parsedElement));
		}

		return Collections.synchronizedList(users);
	}

	public static List<User> parseUsers(final Elements elements) {
		List<Element> parsedElements = parseXmlElements(elements, "users", "user");
		List<User> users = new ArrayList<User>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) users.add(new User(parsedElement));
		}

		return Collections.synchronizedList(users);
	}

	public static User parseUser(final Elements elements) {
		Element parsedElement = parseXmlElement(elements, "user");

		if(parsedElement == null) return null;

		return new User(parsedElement);
	}

	public static Right parseRight(final Elements elements) {
		Element parsedElement = parseXmlElement(elements, "rights"); // BrainHoney uses plural.

		if(parsedElement == null) return null;

		return new Right(parsedElement);
	}

	public static List<Course> parseNestedCourses(final Elements elements) {
		List<Element> parsedElements = parseNestedXmlElements(elements, "course");
		List<Course> courses = new ArrayList<Course>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) courses.add(new Course(parsedElement));
		}

		return Collections.synchronizedList(courses);
	}

	public static List<Course> parseCourses(final Elements elements) {
		List<Element> parsedElements = parseXmlElements(elements, "courses", "course");
		List<Course> courses = new ArrayList<Course>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) courses.add(new Course(parsedElement));
		}

		return Collections.synchronizedList(courses);
	}

	public static Course parseCourse(final Elements elements) {
		Element parsedElement = parseXmlElement(elements, "course");

		if(parsedElement == null) return null;

		return new Course(parsedElement);
	}

	public static List<Section> parseNestedSections(final Elements elements) {
		List<Element> parsedElements = parseNestedXmlElements(elements, "section");
		List<Section> sections = new ArrayList<Section>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) sections.add(new Section(parsedElement));
		}

		return Collections.synchronizedList(sections);
	}

	public static List<Section> parseSections(final Elements elements) {
		List<Element> parsedElements = parseXmlElements(elements, "sections", "section");
		List<Section> sections = new ArrayList<Section>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) sections.add(new Section(parsedElement));
		}

		return Collections.synchronizedList(sections);
	}

	public static Section parseSection(final Elements elements) {
		Element parsedElement = parseXmlElement(elements, "section");

		if(parsedElement == null) return null;

		return new Section(parsedElement);
	}

	public static List<Enrollment> parseNestedEnrollments(final Elements elements) {
		List<Element> parsedElements = parseNestedXmlElements(elements, "enrollment");
		List<Enrollment> enrollments = new ArrayList<Enrollment>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) enrollments.add(new Enrollment(parsedElement));
		}

		return Collections.synchronizedList(enrollments);
	}

	public static List<Enrollment> parseEnrollments(final Elements elements) {
		List<Element> parsedElements = parseXmlElements(elements, "enrollments", "enrollment");
		List<Enrollment> enrollments = new ArrayList<Enrollment>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) enrollments.add(new Enrollment(parsedElement));
		}

		return Collections.synchronizedList(enrollments);
	}

	public static Enrollment parseEnrollment(final Elements elements) {
		Element parsedElement = parseXmlElement(elements, "enrollment");

		if(parsedElement == null) return null;

		return new Enrollment(parsedElement);
	}

	public static List<Group> parseNestedGroups(final Elements elements) {
		List<Element> parsedElements = parseNestedXmlElements(elements, "group");
		List<Group> groups = new ArrayList<Group>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) groups.add(new Group(parsedElement));
		}

		return Collections.synchronizedList(groups);
	}

	public static List<Group> parseGroups(final Elements elements) {
		List<Element> parsedElements = parseXmlElements(elements, "groups", "group");
		List<Group> groups = new ArrayList<Group>();

		for(Element parsedElement : parsedElements) {
			if(parsedElement != null) groups.add(new Group(parsedElement));
		}

		return Collections.synchronizedList(groups);
	}

	public static Group parseGroup(final Elements elements) {
		Element parsedElement = parseXmlElement(elements, "group");

		if(parsedElement == null) return null;

		return new Group(parsedElement);
	}
}