package org.hashtree.jbrainhoney.network.http;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CookieStore;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.cookie.Cookie;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.FileEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.CoreConnectionPNames;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.util.EntityUtils;
import org.hashtree.jbrainhoney.network.http.request.Body;
import org.hashtree.jbrainhoney.network.http.request.ByteArrayBody;
import org.hashtree.jbrainhoney.network.http.request.GetRequest;
import org.hashtree.jbrainhoney.network.http.request.PostRequest;

/** A thread safe implementation of an HttpClient. */
public final class ThreadSafeHttpClient implements HttpClient {
	private final ThreadSafeClientConnManager connectionManager;
	private final CookieStore cookieStore;

	private ThreadSafeHttpClient(final Builder builder) {
		connectionManager = new ThreadSafeClientConnManager();
		connectionManager.setMaxTotal(75);

		cookieStore = new BasicCookieStore();
	}

	private static HttpEntity createHttpEntity(final Body body) {
		if(body == null) throw new IllegalArgumentException("Expected body to not be null.");

		HttpEntity entity = null;

		if(body instanceof FileBody) entity = new FileEntity((File)body.getValue(), body.getMimeType());
		else if(body instanceof ByteArrayBody) entity = new ByteArrayEntity((byte[])body.getValue());
		else {
			try {
				entity = new StringEntity((String)body.getValue(), body.getMimeType(), "UTF-8");
			} catch(UnsupportedEncodingException exception) {
				throw new HttpException(exception);
			}
		}

		return entity;
	}

	@Override
	public String get(final GetRequest getRequest) {
		if(getRequest == null) throw new IllegalArgumentException("Expected getRequest to not be null.");
		else if(getRequest.getUri() == null)
			throw new IllegalArgumentException("Expected getRequest.uri to not be null.");

		HttpGet get = new HttpGet(getRequest.getUri().toString());

		return execute(get);
	}

	@Override
	public String post(final PostRequest postRequest) throws IllegalArgumentException, HttpException {
		if(postRequest == null) throw new IllegalArgumentException("Expected postRequest to not be null.");
		else if(postRequest.getUri() == null)
			throw new IllegalArgumentException("Expected postRequest.uri to not be null.");
		else if(postRequest.getBody() == null)
			throw new IllegalArgumentException("Expected postRequest.body to not be null.");

		HttpPost post = new HttpPost(postRequest.getUri().toString());
		post.setEntity(createHttpEntity(postRequest.getBody()));

		return execute(post);
	}

	@Override
	public void close() {
		clearCookies();
		connectionManager.shutdown();
	}

	@Override
	public boolean hasCookies() {
		cookieStore.clearExpired(new Date());

		return (cookieStore.getCookies() != null && cookieStore.getCookies().size() > 0);
	}

	@Override
	public List<Cookie> getCookies() {
		cookieStore.clearExpired(new Date());

		return Collections.unmodifiableList(new ArrayList<Cookie>(cookieStore.getCookies()));
	}

	@Override
	public ThreadSafeHttpClient clearCookies() {
		cookieStore.clear();

		return this;
	}

	private String execute(final HttpUriRequest method) throws HttpException {
		// Create client.
		DefaultHttpClient client = new DefaultHttpClient(connectionManager);
		client.getParams().setParameter(CoreProtocolPNames.PROTOCOL_VERSION, HttpVersion.HTTP_1_1);
		client.getParams().setParameter(CoreProtocolPNames.USER_AGENT, "jbrainhoney");
		client.getParams().setParameter(CoreConnectionPNames.CONNECTION_TIMEOUT, 30000);
		client.setCookieStore(cookieStore);

		String responseBody = null;
		HttpEntity entity = null;

		try {
			HttpResponse response = client.execute(method);

			if(response.getStatusLine().getStatusCode() != 200)
				throw new HttpException("Expected 200 status code. Received " + response.getStatusLine().getStatusCode() + " status code.");

			entity = response.getEntity();

			if(entity == null) throw new HttpException("Expected a response entity.");

			responseBody = EntityUtils.toString(entity);
		} catch(ClientProtocolException exception) {
			method.abort();
			throw new HttpException(exception);
		} catch(IOException exception) {
			method.abort();
			throw new HttpException(exception);
		} finally {
			try{EntityUtils.consume(entity);}catch(java.lang.Exception exception){throw new HttpException(exception);}
		}

		return responseBody;
	}

	@Override
	protected void finalize() throws Throwable {
		try {
			close();
		} finally {
			super.finalize();
		}
	}

	/** A builder of immutable ThreadSafeHttpClient instances. */
	public final static class Builder implements HttpClient.Builder {
		public Builder() {

		}

		@Override
		@SuppressWarnings("synthetic-access")
		public ThreadSafeHttpClient build() {
			return new ThreadSafeHttpClient(this);
		}
	}
}